;
; Creation par AC le 16/06/2005 a Paris,
; suite a discussion serieuse avec B. Quenez
;
; ----------------------------------------------------------
; Ce module calcule une STDDEV raisonnable meme si le signal
; comprend une derive, ou des structures (crenneaux, sinusoides, ...)
;
; NB: lorsque le signal est un bruit gaussien sur une base plate,
;     cette approche rend la meme valeur que STDDEV
;
; Dans un signal affecter par une derive (de forme quelconque),
; dans lequel est ajouter un signal
; on peut utiliser l'approche SHIFT afin de retirer
; ET la derive ET la signature du signal.
;
; Cette approche est plus facile a utiliser que passer par le spectre (FFT)
;
; Dans le cas d'un signal crenaux, ceci ajoute bien sur
; des glitchs de la hauteur des marches. Mais si ce nombre
; de glitch est "petit" par rapport au nombre total de points,
; cette approche ne necessite meme pas de gerer ces glitchs
;
; ----------------------------------------------------------
; Historique des modifications:
; AC le 02/08/05 :
; - mise a jour de la doc et des commentaires, et gestion des options
; - creation de qlq exemples (pro TEST_AC_STDDEV)
;
; ----------------------------------------------------------
;
function AC_STDDEV, signal, display=display, verbose=verbose, $
                    debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('AC_STDDEV')
;
if (N_PARAMS() lt 1) then begin
    print, mess.fatal+'missing arguments'
    help=1
endif
;
if (N_ELEMENTS(signal) LT 10) then begin
    print, mess.fatal+'too few points in input signal ...'
    return, -1
    endif
if (N_ELEMENTS(signal) LT 100) then begin
    print, mess.warning+'few points in input signal ...'
endif
;
if KEYWORD_SET(help) then begin
    print, 'function AC_STDDEV, signal, display=display, verbose=verbose, $'
    print, '             debug=debug, test=test, help=help'
    print, ''
    print, 'CALLING SEQUENCE : print, AC_STDDEV(signal)'
    return, -1
endif
;
; on retire la derive et autres signaux 
;
processed=(signal-SHIFT(signal,1))(1:*)
;
; on utilise la STDDEV "de base"
;
resultat=STDDEV(processed)
;
; Des fois, il est pertinenet de jeter un oeil aux signaux,
; en particulier 
;
if KEYWORD_SET(display) then begin
    !p.multi=[0,0,2]
    plot, signal, /ynozero, title='Original signal'
    plot, processed, /ynozero, title='Processed signal'
    AC_DELAIS
endif
;
if KEYWORD_SET(debug) then begin
    HELP
endif
;
if KEYWORD_SET(test) then STOP
;
return, resultat/SQRT(2.)
;
end
;
; -----------------------------
;
pro TEST_AC_STDDEV
;
print, 'First tests with a square pulse and Uniform noise'
print, 'In that case, we need a large number of points to converge ...'
print, 'Such <<problem>> is expected since we assume Gaussiant noise'
;
nbp0=100
for i=1,3 do begin
    nbp1=nbp0*10L^i
    s1=DEMO_STEP(time=nbp1, step=1,flux=6,jumps_up=0.1*(1+FINDGEN(6)), width=0.05, /quiet)
    b1=RANDOMU(seed,nbp1)
    s1=s1+b1
    a=AC_STDDEV(s1,/display)
    print, 'Nbp: ', nbp1, ', Reference: ', STDDEV(b1), $
      ', STDDEV: ', STDDEV(s1),  ', AC STDDEV: ', a
endfor
;
print, 'Second tests with same square pulse but Gaussian (normal) noise'
;
nbp0=100
for i=1,3 do begin
    nbp1=nbp0*10L^i
    s1=DEMO_STEP(time=nbp1, step=1,flux=6,jumps_up=0.1*(1+FINDGEN(6)), width=0.05, /quiet)
    b1=RANDOMN(seed,nbp1)
    s1=s1+b1
    a=AC_STDDEV(s1,/display)
    print, 'Nbp: ', nbp1, ', Reference: ', STDDEV(b1), $
      ', STDDEV: ', STDDEV(s1),  ', AC STDDEV: ', a
endfor
;
print, 'Third tests with drift, same square pulse but Gaussian (normal) noise'
;
nbp0=100
for i=1,3 do begin
    nbp1=nbp0*10L^i
    s1=DEMO_STEP(time=nbp1, step=1,flux=6,jumps_up=0.1*(1+FINDGEN(6)), width=0.05, /quiet)
    b1=RANDOMN(seed,nbp1)
    s1=s1+b1+10.*FINDGEN(nbp1)/FLOAT(nbp1-1)
    a=AC_STDDEV(s1,/display)
    print, 'Nbp: ', nbp1, ', Reference: ', STDDEV(b1), $
      ', STDDEV: ', STDDEV(s1),  ', AC STDDEV: ', a
endfor
; ---------------------
;
nbp=1000
bruit=RANDOMU(seed,nbp)
for i=0, 5 do begin
    exposant=exp((i-2)/alog10(exp(1)))
    signal=bruit+FINDGEN(nbp)*exposant
    print, 'Exposant:', exposant, ', Reference:', STDDEV(bruit), $
      ', STDDEV:', STDDEV(signal),  ', AC STDDEV:', AC_STDDEV(signal)
endfor
;
stop
;
end
